package tech.nevets;

import net.miginfocom.swing.MigLayout;
import tech.nevets.util.CastUtils;
import tech.nevets.util.DataUtils;
import tech.nevets.util.Employee;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

public class STEmployeeHours extends JFrame {

    private JPanel contentPane;
    private static final ArrayList<Employee> employeeList = new ArrayList<>();
    private static final DataUtils<Employee> EMPLOYEE_DATA_UTILS = new DataUtils<>(Employee.class);
    private static final CastUtils<Employee> EMPLOYEE_CAST_UTILS = new CastUtils<>(Employee.class);

    JLabel idLabel;
    JTextField idField;
    JButton submitEmpId;
    JLabel empFirstName;
    JLabel empLastName;
    JLabel empHours;

    JLabel addHoursLabel;
    JTextField addHoursField;
    JButton addHoursButton;

    public STEmployeeHours() {
        contentPane = new JPanel();
        employeeList.clear();
        employeeList.addAll(List.of(EMPLOYEE_CAST_UTILS.castArray(EMPLOYEE_DATA_UTILS.getArrayFromData("EmployeeData"))));
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                EMPLOYEE_DATA_UTILS.saveToDataArray(EMPLOYEE_CAST_UTILS.castArray(employeeList.toArray()), "EmployeeData", Employee[].class);
            }
        });
        setContentPane(contentPane);
        contentPane.setBorder(new EmptyBorder(1, 1, 1, 1));
        MigLayout layout = new MigLayout(
                "",
                "[][fill][]",
                "[][][][][][]"
        );
        contentPane.setLayout(layout);

        idLabel = new JLabel("Employee Id:");
        contentPane.add(idLabel, "cell 0 0");
        idField = new JTextField();
        contentPane.add(idField, "cell 1 0,w 150");

        AtomicInteger empIdCache = new AtomicInteger();
        submitEmpId = new JButton("Submit");
        submitEmpId.addActionListener(al -> {
            empIdCache.set(Integer.parseInt(idField.getText()));
            boolean isFound = false;
            for (Employee e : employeeList) {
                if (idField.getText().equals(String.valueOf(e.getEmployeeId()))) {
                    isFound = true;
                    idLabel.setText("Employee Id:");
                    updateUI(e);
                    break;
                }
            }
            if (!isFound) {
                idLabel.setText("User Not Found");
            }
        });
        contentPane.add(submitEmpId, "cell 2 0");

        empFirstName = new JLabel("First Name: ");
        contentPane.add(empFirstName, "cell 1 1");
        empLastName = new JLabel("Last Name: ");
        contentPane.add(empLastName, "cell 1 2");
        empHours = new JLabel("Total Hours: ");
        contentPane.add(empHours, "cell 1 3");

        addHoursLabel = new JLabel("Hours To Add:");
        contentPane.add(addHoursLabel, "cell 0 4");
        addHoursField = new JTextField();
        contentPane.add(addHoursField, "cell 1 4");
        addHoursButton = new JButton("Add");
        addHoursButton.addActionListener(al -> {
            updateEmpHours(empIdCache.get(), Integer.parseInt(addHoursField.getText()));
            updateUI(empIdCache.get());
            addHoursField.setText("");
        });
        contentPane.add(addHoursButton, "cell 2 4,grow");

        JButton newUserBtn = new JButton("New User");
        newUserBtn.addActionListener(al -> {
            AddUserFrame newUserFrame = new AddUserFrame();
            newUserFrame.setLocationRelativeTo(null);
            newUserFrame.pack();
            newUserFrame.setVisible(true);
        });
        contentPane.add(newUserBtn, "cell 1 5,grow");
    }

    private void updateUI(Employee e) {
        empFirstName.setText("First Name: " + e.getEmployeeFirstName());
        empLastName.setText("Last Name: " + e.getEmployeeLastName());
        empHours.setText("Total Hours: " + e.getEmployeeHours());
    }

    private void updateUI(int empId) {
        for (Employee e : employeeList) {
            if (e.getEmployeeId() == empId) {
                empFirstName.setText("First Name: " + e.getEmployeeFirstName());
                empLastName.setText("Last Name: " + e.getEmployeeLastName());
                empHours.setText("Total Hours: " + e.getEmployeeHours());
            }
        }
    }

    private void updateEmpHours(int id, int hoursToAdd) {
        for (Employee e : employeeList) {
            if (e.getEmployeeId() == id) {
                e.addHours(hoursToAdd);
            }
        }
    }

    public static void main(String[] args) {
        STEmployeeHours frame = new STEmployeeHours();
        frame.setBounds(0, 0, 860, 480);
        frame.setLocationRelativeTo(null);
        frame.pack();
        frame.setVisible(true);

    }

    private class AddUserFrame extends JFrame {
        public AddUserFrame() {
            setResizable(false);
            JPanel auContentPane = new JPanel();
            setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            setContentPane(auContentPane);
            auContentPane.setBorder(new EmptyBorder(1, 1, 1, 1));
            MigLayout layout = new MigLayout(
                    "",
                    "[][fill][]",
                    "[][][][][][]"
            );
            auContentPane.setLayout(layout);

            JLabel errorLabel = new JLabel("");
            auContentPane.add(errorLabel, "cell 0 5");

            JLabel idLabel = new JLabel("User Id:");
            auContentPane.add(idLabel, "cell 0 1");
            JTextField idField = new JTextField();
            auContentPane.add(idField, "cell 1 1");
            JLabel firstNameLabel = new JLabel("First Name:");
            auContentPane.add(firstNameLabel, "cell 0 2");
            JTextField firstNameField = new JTextField();
            auContentPane.add(firstNameField, "cell 1 2");
            JLabel lastNameLabel = new JLabel("Last Name:");
            auContentPane.add(lastNameLabel, "cell 0 3");
            JTextField lastNameField = new JTextField();
            auContentPane.add(lastNameField, "cell 1 3");
            JButton submitButton = new JButton("Create");
            submitButton.addActionListener(al -> {
                boolean idExists = false;
                for (Employee e : employeeList) {
                    if (e.getEmployeeId() == Integer.parseInt(idField.getText())) {
                        errorLabel.setText("User with that id already exists");
                         idExists = true;
                        break;
                    }
                }
                if (!idExists) {
                    Employee e = new Employee(Integer.parseInt(idField.getText()), firstNameField.getText(), lastNameField.getText());
                    employeeList.add(e);
                    this.dispose();
                }
            });
            auContentPane.add(submitButton, "cell 1 4");
            JButton cancelButton = new JButton("Cancel");
            cancelButton.addActionListener(al -> {
                this.dispose();
            });
            auContentPane.add(cancelButton, "cell 2 4");
        }
    }
}
