package tech.nevets.util;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonArray;

import java.io.*;

public class DataUtils<T> {
    private Class<T> clazz;
    private final Gson gson = new GsonBuilder().setPrettyPrinting().create();

    public DataUtils(Class<T> clazz) {
        this.clazz = clazz;
        File dataDir = new File("./data/");
        if (!dataDir.exists()) {
            dataDir.mkdirs();
        }
    }

    public void saveToDataArray(T[] array, String fileName, Class<T[]> clazzLocal) {
        String json = gson.toJson(array, clazzLocal);
        writeJsonToFile(json, fileName);
    }

    public void saveToDataObject(T data, String fileName) {
        String json = gson.toJson(data, clazz);
        writeJsonToFile(json, fileName);
    }

    private void writeJsonToFile(String json, String fileName) {
        try {
            FileWriter fw = new FileWriter("./data/" + fileName + ".json");
            fw.write(json);
            fw.flush();
            fw.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public T[] getArrayFromData(String fileName) {
        File file = new File("./data/" + fileName + ".json");
        JsonArray ja;
        try {
            if (!file.exists()) {
                file.createNewFile();
            }
            BufferedReader br = new BufferedReader(new FileReader(file));
            ja = gson.fromJson(br, JsonArray.class);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }

        if (ja == null) {
            ja = new JsonArray();
        }

        Object[] dataArray = new Object[ja.size()];

        for (int i = 0; i < ja.size(); i++) {
            dataArray[i] = gson.fromJson(ja.get(i).getAsJsonObject(), clazz);
        }

        return new CastUtils<>(clazz).castArray(dataArray);
    }

    public T getObjectFromData(String fileName) {
        try {
            BufferedReader br = new BufferedReader(new FileReader("./data/" + fileName + ".json"));
            return gson.fromJson(br, clazz);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }
}
